import argparse
import pkg_resources
import sys
from operatorcourier import api


def main():
    """Generate the CLI bits
    """
    try:
        parser = _CliParser()
        parser.parse()
    except Exception as e:  # Exception raised to CLI should not be thrown to users,
        sys.exit(str(e))    # it should just be captured by logs


class _CliParser():
    """Class that generates the command line bits for the operator-courier cli tool
    """

    def __init__(self):
        pass

    def parse(self):
        """Parse generates and evaluates the command level parser
        """
        parser = argparse.ArgumentParser(
            description='Build, verify and push operator bundles into '
                        'external app registry',
            usage='''operator-courier <command> [<args>]

These are the commands you can use:
    verify      Create a bundle and test it for correctness.
    push        Create a bundle, test it, and push it to an app registry.
    nest        Take a flat to-be-bundled directory and version nest it.
''')
        try:
            __version__ = pkg_resources.get_distribution('operator-courier').version
        except Exception:
            __version__ = 'unknown'

        parser.add_argument('command', help='Subcommand to run')
        parser.add_argument('-v', '--version', help='Show the current version of operator-courier',
                            action='version', version=__version__)

        args = parser.parse_args(sys.argv[1:2])
        if not hasattr(self, args.command):
            parser.error('Unrecognized command')

        getattr(self, args.command)()

    # Parse the verify command
    def verify(self):
        parser = argparse.ArgumentParser(
            description='Build and verify an operator bundle to test')
        parser.add_argument('source_dir', help='Path of your directory of yaml '
                            'files to bundle. Either set this or '
                            'use the files argument for bundle data.')
        parser.add_argument('--ui_validate_io',
                            help='Validate bundle for operatorhub.io UI',
                            action='store_true')
        parser.add_argument('--validation-output', dest='validation_output',
                            help='A file to write validation warnings and errors to'
                                 'in JSON format')
        args, leftovers = parser.parse_known_args(sys.argv[2:])
        api.build_and_verify(source_dir=args.source_dir,
                             ui_validate_io=args.ui_validate_io,
                             validation_output=args.validation_output)

    # Parse the push command
    def push(self):
        parser = argparse.ArgumentParser(
            description='Build, verify and push an operator bundle '
                        'into external app registry.')
        parser.add_argument('source_dir',
                            help='Path of your directory of yaml files to bundle.')
        parser.add_argument('namespace',
                            help='Name of the Quay namespace to push operator to.')
        parser.add_argument('repository',
                            help='Application repository name '
                                 'the application is bundled for.')
        parser.add_argument('release',
                            help='The release version of the bundle.')
        parser.add_argument('token', help='Authorization token for Quay api.')
        parser.add_argument('--validation-output', dest='validation_output',
                            help='A file to write validation warnings and errors to'
                                 'in JSON format')
        args, leftovers = parser.parse_known_args(sys.argv[2:])
        api.build_verify_and_push(args.namespace,
                                  args.repository,
                                  args.release,
                                  args.token,
                                  source_dir=args.source_dir,
                                  validation_output=args.validation_output)

    # Parse the nest command
    def nest(self):
        parser = argparse.ArgumentParser(
            description='Take a flat bundle directory and version nest it '
                        'to eventually create an operator-registry image.')
        parser.add_argument('source_dir',
                            help='Path of your directory of yaml files to bundle.')
        parser.add_argument('registry_dir',
                            help='Path of your directory to be populated. '
                                 'If directory does not exist, it will be created.')

        args, leftovers = parser.parse_known_args(sys.argv[2:])
        api.nest(args.source_dir, args.registry_dir)



import yaml
import operatorcourier.identify as identify


class BuildCmd():
    def _get_empty_bundle(self):
        return dict(
            data=dict(
                customResourceDefinitions=[],
                clusterServiceVersions=[],
                packages=[],
            )
        )

    def _get_field_entry(self, yamlContent):
        yaml_type = identify.get_operator_artifact_type(yamlContent)
        return yaml_type[0:1].lower() + yaml_type[1:] + 's'

    def _updateBundle(self, operatorBundle, yamlContent):
        field_entry = operatorBundle["data"][self._get_field_entry(yamlContent)]
        field_entry.append(yaml.safe_load(yamlContent))
        return operatorBundle

    def build_bundle(self, strings):
        """build_bundle takes an array of yaml files and generates a 'bundle'
        with those yaml files generated in the bundle format.

        :param strings: Array of yaml strings to bundle
        """
        bundle = self._get_empty_bundle()
        for item in strings:
            bundle = self._updateBundle(bundle, item)

        return bundle



import pytest
import operatorcourier.identify as identify
from testfixtures import LogCapture


@pytest.mark.parametrize('fname,expected', [
    ("tests/test_files/csv.yaml", "ClusterServiceVersion"),
    ("tests/test_files/crd.yaml", "CustomResourceDefinition"),
    ("tests/test_files/package.yaml", "Package"),
])
def test_get_operator_artifact_type(fname, expected):
    with open(fname) as f:
        yaml = f.read()
        assert identify.get_operator_artifact_type(yaml) == expected


@pytest.mark.parametrize('fname', [
    ("tests/test_files/invalid.yaml"),
    ("tests/test_files/empty.yaml"),
])
def test_get_operator_artifact_type_assertions(fname):
    with open(fname) as f:
        yaml = f.read()
    with pytest.raises(ValueError) as e, LogCapture() as logs:
        identify.get_operator_artifact_type(yaml)

    logs.check(('operatorcourier.identify',
                'ERROR',
                'Courier requires valid CSV, CRD, and Package files'),)
    assert 'Courier requires valid CSV, CRD, and Package files' == str(e.value)


@pytest.mark.parametrize('fname', [
    "tests/test_files/invalid.malformed.parser.error.yaml",
    "tests/test_files/invalid.malformed.scanner.error.yaml",
])
def test_get_operator_artifact_type_with_invalid_yaml(fname):
    with open(fname) as f:
        yaml = f.read()

    with pytest.raises(ValueError) as e, LogCapture() as logs:
        identify.get_operator_artifact_type(yaml)

    logs.check(('operatorcourier.identify',
                'ERROR',
                'Courier requires valid input YAML files'),)
    assert 'Courier requires valid input YAML files' == str(e.value)



from yaml import safe_load
from yaml import MarkedYAMLError
import logging

logger = logging.getLogger(__name__)


def get_operator_artifact_type(operatorArtifactString):
    """get_operator_artifact_type takes a yaml string and determines if it is
    one of the expected bundle types: ClusterServiceVersion,
    CustomResourceDefinition, or Package.

    :param operatorArtifactString: Yaml string to type check
    """
    try:
        operatorArtifact = safe_load(operatorArtifactString)
    except MarkedYAMLError:
        msg = "Courier requires valid input YAML files"
        logger.error(msg)
        raise ValueError(msg)
    else:
        artifact_type, artifact_name = None, None
        if isinstance(operatorArtifact, dict):
            if "packageName" in operatorArtifact:
                artifact_type = "Package"
                artifact_name = operatorArtifact['packageName']
            elif operatorArtifact.get("kind") in ("ClusterServiceVersion",
                                                  "CustomResourceDefinition"):
                artifact_type = operatorArtifact["kind"]
                artifact_name = operatorArtifact['metadata']['name']
            if artifact_type is not None:
                logger.info('Parsed %s: %s', artifact_type, artifact_name)
                return artifact_type

        msg = 'Courier requires valid CSV, CRD, and Package files'
        logger.error(msg)
        raise ValueError(msg)



import yaml
import pytest
from operatorcourier.validate import ValidateCmd
from operatorcourier.format import unformat_bundle


@pytest.mark.parametrize('bundle,expected_validation_results_dict', [
    ("tests/test_files/bundles/verification/valid.bundle.yaml",
        {'errors': [], 'warnings': ['csv spec.icon not defined']}),
    ("tests/test_files/bundles/verification/nocrd.valid.bundle.yaml",
        {'errors': [], 'warnings': ['csv spec.icon not defined',
         'csv spec.maturity not defined']}),
])
def test_valid_bundles(bundle, expected_validation_results_dict):
    valid, validation_results_dict = get_validation_results(bundle)
    assert valid is True
    assert validation_results_dict == expected_validation_results_dict


@pytest.mark.parametrize('bundle,expected_validation_results_dict', [
    ("tests/test_files/bundles/verification/nopkg.invalid.bundle.yaml",
        {'errors': ['Bundle does not contain any packages.'],
            'warnings': ['csv spec.icon not defined']}),
    ("tests/test_files/bundles/verification/no-data-key.bundle.yaml",
        {'errors': ['Bundle does not contain any clusterServiceVersions.',
                    'Bundle does not contain any packages.'], 'warnings': []}),
])
def test_invalid_bundle(bundle, expected_validation_results_dict):
    valid, validation_results_dict = get_validation_results(bundle)
    assert valid is False
    assert validation_results_dict == expected_validation_results_dict


@pytest.mark.parametrize('bundle,expected_validation_results_dict', [
        ("tests/test_files/bundles/verification/valid.bundle.yaml",
            {'errors': [], 'warnings': ['csv spec.icon not defined',
                                        'csv spec.icon not defined. Without this field, '
                                        'the operator will display a default operator '
                                        'framework icon.']}), ])
def test_ui_valid_bundle_io(bundle, expected_validation_results_dict):
    valid, validation_results_dict = get_ui_validation_results(bundle)
    assert valid is True
    assert validation_results_dict == expected_validation_results_dict


def get_ui_validation_results(bundle):
    return ValidateCmd(ui_validate_io=True).validate(get_bundle(bundle))


def get_validation_results(bundle):
    return ValidateCmd().validate(get_bundle(bundle))


def get_bundle(bundle):
    with open(bundle) as f:
        bundle = yaml.safe_load(f)
    return unformat_bundle(bundle)



import logging
import json
import re

import validators as v

from .const_io import (
    general_required_fields,
    metadata_required_fields,
    metadata_annotations_required_fields,
    spec_required_fields)

logger = logging.getLogger(__name__)


class ValidateCmd():
    dataKey = "data"
    crdKey = "customResourceDefinitions"
    csvKey = "clusterServiceVersions"
    pkgsKey = "packages"

    def __init__(self, ui_validate_io=False):
        self.ui_validate_io = ui_validate_io
        self.validation_json = dict(
            warnings=[],
            errors=[],
        )
        pass

    def _log_warning(self, message, *args, **kwargs):
        """_log_warning prints the message to the logger as a warning
        and appends it to a dictionary that can be printed to the
        screen for automation purposes
         :param message: The message to log
        """
        self.validation_json['warnings'].append(message % args)
        logger.warning(message, *args, **kwargs)

    def _log_error(self, message, *args, **kwargs):
        """_log_error prints the message to the logger as an error
        and appends it to a dictionary that can be printed to the
        screen for automation purposes
         :param message: The message to log
        """
        self.validation_json['errors'].append(message % args)
        logger.error(message, *args, **kwargs)

    def validate(self, bundle):
        """validate takes a bundle as a dictionary and returns a boolean value that
        describes if the bundle is valid. It also logs verification information when
        there are invalid sections in the bundle.

        :param bundle: Dictionary of bundle value
        """
        logger.info("Validating bundle.")

        return self._bundle_validation(bundle), self.validation_json

    def _bundle_validation(self, bundle):
        validationDict = dict()

        if self.dataKey not in bundle:
            self._log_error("Bundle does not contain base data field.")
            # break here because there's nothing else to learn
            return False

        bundleData = bundle[self.dataKey]

        validationDict[self.crdKey] = self._type_validation(bundleData, self.crdKey,
                                                            self._crd_validation, False)
        validationDict[self.csvKey] = self._type_validation(bundleData, self.csvKey,
                                                            self._csv_validation, True)
        validationDict[self.pkgsKey] = self._type_validation(bundleData, self.pkgsKey,
                                                             self._pkgs_validation, True)
        if self.ui_validate_io:
            validationDict[self.csvKey] &= self._type_validation(
                bundleData, self.csvKey, self._ui_validation_io, True)

        valid = True
        for key, value in validationDict.items():
            if value is False:
                valid = False

        return valid

    def _crd_validation(self, bundleData):
        logger.info("Validating custom resource definitions.")
        valid = True

        crds = bundleData[self.crdKey]

        for crd in crds:
            if "metadata" in crd:
                if "name" in crd["metadata"]:
                    logger.info("Evaluating crd %s", crd["metadata"]["name"])
                else:
                    self._log_error("crd metadata.name not defined.")
                    valid = False
            else:
                self._log_error("crd metadata not defined.")
                valid = False

            if "apiVersion" not in crd:
                self._log_error("crd apiVersion not defined.")
                valid = False

            if "spec" not in crd:
                self._log_error("crd spec not defined.")
                valid = False

        return valid

    def _csv_validation(self, bundleData):
        valid = True
        logger.info("Validating cluster service versions.")

        csvs = bundleData[self.csvKey]

        for csv in csvs:
            if "metadata" in csv:
                if self._csv_metadata_validation(csv["metadata"]) is False:
                    valid = False
            else:
                self._log_error("csv metadata not defined.")
                valid = False

            if "apiVersion" not in csv:
                self._log_error("csv apiVersion not defined.")
                valid = False

            if "spec" in csv:
                if self._csv_spec_validation(csv["spec"], bundleData) is False:
                    valid = False
            else:
                self._log_error("csv spec not defined.")
                valid = False

        return valid

    def _csv_spec_validation(self, spec, bundleData):
        valid = True

        warnSpecList = ["displayName", "description", "icon",
                        "version", "provider", "maturity"]

        for item in warnSpecList:
            if item not in spec:
                self._log_warning("csv spec.%s not defined" % item)

        if "installModes" not in spec:
            self._log_error("csv spec.installModes not defined")
            valid = False

        if "install" not in spec:
            self._log_error("csv spec.install not defined")
            valid = False

        if "customresourcedefinitions" in spec:
            customresourcedefinitions = spec["customresourcedefinitions"]

            crdList = []
            for crd in bundleData[self.crdKey]:
                try:
                    name = crd["metadata"]["name"]
                    crdList.append(name)
                except KeyError:
                    pass

            if "owned" not in customresourcedefinitions:
                self._log_error("spec.customresourcedefinitions.owned"
                                "not defined for csv")
                valid = False
            else:
                for crd in customresourcedefinitions["owned"]:
                    if "name" not in crd:
                        self._log_error("name not defined for item in "
                                        "spec.customresourcedefinitions.")
                        valid = False
                    else:
                        if crd["name"] not in crdList:
                            self._log_error("custom resource definition referenced "
                                            "in csv not defined in root list of crds")

        return valid

    def _csv_metadata_validation(self, metadata):
        valid = True

        if "name" in metadata:
            logger.info("Evaluating csv %s", metadata["name"])
        else:
            self._log_error("csv metadata.name not defined.")
            valid = False

        if "annotations" in metadata:
            annotations = metadata["annotations"]

            annotationList = ["categories", "description",
                              "containerImage", "createdAt", "support"]

            for item in annotationList:
                if item not in annotations:
                    self._log_warning("csv metadata.annotations.%s not defined" % item)

            # check certified value's type in particular. should be string, not bool
            if "certified" not in annotations:
                self._log_warning("csv metadata.annotations.certified not defined.")
            else:
                isString = isinstance(annotations["certified"], str)
                if not isString:
                    self._log_error("metadata.annotations.certified is not of type"
                                    "string")
                    valid = False

            # if alm-examples is defined, check that its value is valid json
            if "alm-examples" in annotations:
                try:
                    json.loads(annotations["alm-examples"])
                except KeyError:
                    self._log_error("metadata.annotations.alm-examples contains "
                                    "invalid json string")
                    valid = False

        else:
            self._log_warning("csv metadata.annotations not defined.")

        return valid

    def _pkgs_validation(self, bundleData):
        valid = True
        logger.info("Validating packages.")

        pkgs = bundleData[self.pkgsKey]

        for pkg in pkgs:
            if "packageName" in pkg:
                logger.info("Evaluating package %s", pkg["packageName"])
            else:
                self._log_error("packageName not defined.")
                valid = False

            if "channels" in pkg:
                channels = pkg["channels"]
                if len(channels) == 0:
                    self._log_error("no package channels defined.")
                    valid = False
                else:
                    csvNames = []
                    for csv in bundleData[self.csvKey]:
                        try:
                            csvName = csv["metadata"]["name"]
                            csvNames.append(csvName)
                        except KeyError:
                            pass
                    for channel in channels:
                        if "name" not in channel:
                            self._log_error("package channel.name not defined.")
                            valid = False

                        if "currentCSV" not in channel:
                            self._log_error("package channel.currentCSV not defined.")
                        else:
                            if channel["currentCSV"] not in csvNames:
                                self._log_error("channel.currentCSV %s is not "
                                                "included in list of csvs",
                                                channel["currentCSV"])
                                valid = False

            else:
                self._log_error("package channels not defined.")
                valid = False

        return valid

    def _type_validation(self, bundleData, typeName, validator, required):
        valid = True
        if typeName in bundleData:
            if len(bundleData[typeName]) != 0:
                valid = validator(bundleData)
            elif required:
                self._log_error("Bundle does not contain any %s." % typeName)
                valid = False
        elif required:
            self._log_error("Bundle does not contain any %s." % typeName)
            valid = False

        return valid

    def _ui_validation_io(self, bundleData):
        valid = True
        logger.info("Validating cluster service versions for operatorhub.io UI.")

        csvs = bundleData[self.csvKey]

        for csv in csvs:
            if self._ui_csv_fields_exist_validation_io(csv) is False:
                self._log_error("UI validation failed to verify required "
                                "fields for operatorhub.io exist.")
                valid = False

            if valid:
                if self._ui_csv_fields_format_validation_io(csv) is False:
                    self._log_error("UI validation failed to verify that required "
                                    "fields for operatorhub.io are properly formatted.")
                    valid = False

        return valid

    def _ui_csv_fields_exist_validation_io(self, csv):
        valid = True

        if "metadata" not in csv:
            self._log_error("csv metadata not defined.")
            valid = False
        else:
            if "name" not in csv["metadata"]:
                self._log_error("csv metadata.name not defined.")
                valid = False
            else:
                logger.info("Evaluating csv %s", csv["metadata"]["name"])

                for field in general_required_fields:
                    if field not in csv:
                        self._log_error("csv %s not defined.", field)
                        valid = False
                        return valid

                for field in metadata_required_fields:
                    if field["field"] not in csv["metadata"]:
                        if field["required"]:
                            self._log_error("csv metadata.%s not defined. %s",
                                            field["field"], field["description"])
                            valid = False
                            return valid
                        else:
                            self._log_warning("csv metadata.%s not defined. %s",
                                              field["field"], field["description"])

                for field in metadata_annotations_required_fields:
                    if field["field"] not in csv["metadata"]["annotations"]:
                        if field["required"]:
                            self._log_error("csv metadata.annotations.%s not defined. %s",
                                            field["field"], field["description"])
                            valid = False
                        else:
                            self._log_warning("csv metadata.annotations.%s not defined."
                                              "%s", field["field"], field["description"])

                for field in spec_required_fields:
                    if field["field"] not in csv["spec"]:
                        if field["required"]:
                            self._log_error("csv spec.%s not defined. %s",
                                            field["field"], field["description"])
                            valid = False
                        else:
                            self._log_warning("csv spec.%s not defined. %s",
                                              field["field"], field["description"])

        return valid

    def _ui_csv_fields_format_validation_io(self, csv):

        def is_url(field):
            if not v.url(field):
                return False
            return True

        def is_email(field):
            if not v.email(field):
                return False
            return True

        def is_version(field):
            pattern1 = re.compile(r'v(\d+\.)(\d+\.)(\d)')
            pattern2 = re.compile(r'(\d+\.)(\d+\.)(\d)')
            return pattern1.match(field) or pattern2.match(field)

        def is_capability_level(field):
            levels = [
                "Basic Install",
                "Seamless Upgrades",
                "Full Lifecycle",
                "Deep Insight",
                "Auto Pilot"
            ]
            return field in levels

        def get_alm_kinds(alm_examples):
            res = []
            for example in alm_examples:
                res.append(example["kind"])
            return res

        valid = True

        spec = csv["spec"]
        provider = spec["provider"]
        annotations = csv["metadata"]["annotations"]

        # alm-examples check based on crd field
        if "customresourcedefinitions" in spec:
            if "owned" in spec["customresourcedefinitions"]:
                crds = spec["customresourcedefinitions"]["owned"]
                if "alm-examples" in annotations:
                    alm_kinds = get_alm_kinds(json.loads(annotations["alm-examples"]))
                    for crd in crds:
                        if crd["kind"] not in alm_kinds:
                            self._log_error("%s CRD does not have an entry in "
                                            "alm-examples - please add such an "
                                            "example CR.", crd["kind"])
                            valid = False
                else:
                    self._log_error("You should have alm-examples for every owned CRD")
                    valid = False

        # provider check
        if isinstance(provider, (dict,)):
            if len(provider) != 1:
                self._log_error("csv.spec.provider should be a singleton list.")
                valid = False
            else:
                if "name" not in provider or len(provider.keys()) != 1:
                    self._log_error("csv.spec.provider element should "
                                    "have a single field \"name\".")
                    valid = False
        else:
            self._log_error("csv.spec.provider should contain a \"name\" field.")
            valid = False

        # maintainers check
        if isinstance(spec["maintainers"], (list,)):
            for maintainer in spec["maintainers"]:
                if "name" not in maintainer or "email" not in maintainer:
                    self._log_error("csv.spec.maintainers element should contain "
                                    "both name and email")
                    valid = False
                else:
                    if not is_email(maintainer["email"]):
                        self._log_error("%s is not a valid email", maintainer["email"])
                        valid = False
        else:
            self._log_error("csv.spec.maintainers must be a list of name & email pairs.")
            valid = False

        # links check
        if isinstance(spec["links"], (list,)):
            for link in spec["links"]:
                if "name" not in link or "url" not in link:
                    self._log_error("csv.spec.links element should contain "
                                    "both name and url")
                    valid = False
                else:
                    if not is_url(link["url"]):
                        self._log_error("%s is not a valid url", link["url"])
                        valid = False
        else:
            self._log_error("csv.spec.links must be a list of name & url pairs.")
            valid = False

        # version check
        if not is_version(spec["version"]):
            self._log_error("spec.version %s is not a valid version "
                            "(example of a valid version is: v1.0.12)",
                            spec["version"])
            valid = False

        # capabilities check
        if not is_capability_level(annotations["capabilities"]):
            self._log_error("metadata.annotations.capabilities %s is not a "
                            "valid capabilities level", annotations["capability"])
            valid = False

        return valid



import pytest
import yaml
from operatorcourier import api
from operatorcourier.format import unformat_bundle


@pytest.mark.parametrize('directory,expected', [
    ("tests/test_files/bundles/api/bundle1",
     "tests/test_files/bundles/api/bundle1/results/bundle.yaml"),
])
def test_make_bundle(directory, expected):
    bundle = api.build_and_verify(source_dir=directory)

    with open(expected, "r") as expected_file:
        expected_bundle = yaml.safe_load(expected_file)
        assert unformat_bundle(bundle) == unformat_bundle(expected_bundle)


@pytest.mark.parametrize('yaml_files,expected', [
    (
        [
            "tests/test_files/bundles/api/bundle1/crd.yml",
            "tests/test_files/bundles/api/bundle1/csv.yaml",
            "tests/test_files/bundles/api/bundle1/packages.yaml"
        ],
        "tests/test_files/bundles/api/bundle1/results/bundle.yaml"
    ),
])
def test_make_bundle_with_yaml_list(yaml_files, expected):
    yamls = []
    for file in yaml_files:
        with open(file, "r") as yaml_file:
            yamls.append(yaml_file.read())

    bundle = api.build_and_verify(yamls=yamls)

    with open(expected, "r") as expected_file:
        expected_bundle = yaml.safe_load(expected_file)
        assert unformat_bundle(bundle) == unformat_bundle(expected_bundle)



import yaml
from operatorcourier.build import BuildCmd


class _literal(str):
    pass


def _literal_presenter(dumper, data):
    return dumper.represent_scalar('tag:yaml.org,2002:str', data, style='|')


def _get_empty_formatted_bundle():
    return dict(
        data=dict(
            customResourceDefinitions='',
            clusterServiceVersions='',
            packages='',
        )
    )


def format_bundle(bundle):
    """
    Converts a bundle object into a push-ready bundle by
    changing list values of 'customResourceDefinitions',
    'clusterServiceVersions', and 'packages' into stringified yaml literals.

    This format is required by the Marketplace backend.

    :param bundle: A bundle object
    """

    formattedBundle = _get_empty_formatted_bundle()

    yaml.add_representer(_literal, _literal_presenter)

    if 'data' not in bundle:
        return formattedBundle

    # Format data fields as string literals to match backend expected format
    if bundle['data'].get('customResourceDefinitions'):
        formattedBundle['data']['customResourceDefinitions'] = _literal(
            yaml.dump(bundle['data']['customResourceDefinitions'],
                      default_flow_style=False))

    if 'clusterServiceVersions' in bundle['data']:
        # Format description and alm-examples
        clusterServiceVersions = []
        for csv in bundle['data']['clusterServiceVersions']:
            if csv.get('metadata', {}).get('annotations', {}).get('alm-examples'):
                csv['metadata']['annotations']['alm-examples'] = _literal(
                    csv['metadata']['annotations']['alm-examples'])

            if csv.get('spec', {}).get('description'):
                csv['spec']['description'] = _literal(csv['spec']['description'])

            clusterServiceVersions.append(csv)

        if clusterServiceVersions:
            formattedBundle['data']['clusterServiceVersions'] = _literal(
                yaml.dump(clusterServiceVersions, default_flow_style=False))

    if bundle['data'].get('packages'):
        formattedBundle['data']['packages'] = _literal(
            yaml.dump(bundle['data']['packages'], default_flow_style=False))

    return formattedBundle


def unformat_bundle(formattedBundle):
    """
    Converts a push-ready bundle into a structured object by changing
    stringified yaml of 'customResourceDefinitions', 'clusterServiceVersions',
    and 'packages' into lists of objects.
    Undoing the format helps simplify bundle validation.

    :param formattedBundle: A push-ready bundle
    """

    bundle = BuildCmd()._get_empty_bundle()

    if 'data' not in formattedBundle:
        return bundle

    if 'customResourceDefinitions' in formattedBundle['data']:
        customResourceDefinitions = yaml.safe_load(
            formattedBundle['data']['customResourceDefinitions'])
        if customResourceDefinitions:
            bundle['data']['customResourceDefinitions'] = customResourceDefinitions

    if 'clusterServiceVersions' in formattedBundle['data']:
        clusterServiceVersions = yaml.safe_load(
            formattedBundle['data']['clusterServiceVersions'])
        if clusterServiceVersions:
            bundle['data']['clusterServiceVersions'] = clusterServiceVersions

    if 'packages' in formattedBundle['data']:
        packages = yaml.safe_load(formattedBundle['data']['packages'])
        if packages:
            bundle['data']['packages'] = packages

    return bundle



from operatorcourier.build import BuildCmd


def test_create_bundle():
    paths = [
        "tests/test_files/csv.yaml",
        "tests/test_files/crd.yaml",
        "tests/test_files/package.yaml"
    ]
    yamls = []
    for path in paths:
        with open(path) as f:
            yamls.append(f.read())

    bundle = BuildCmd().build_bundle(yamls)
    assert bool(bundle["data"]["packages"]) is True
    assert bool(bundle["data"]["clusterServiceVersions"]) is True
    assert bool(bundle["data"]["customResourceDefinitions"]) is True



import base64
import requests
import tarfile
import logging
from tempfile import TemporaryDirectory

logger = logging.getLogger(__name__)


class PushCmd():
    name = 'push'

    def __init__(self):
        pass

    def push(self, bundle_dir, namespace, repository, release, auth_token):
        """Push takes a bundle and pushes it to the specified app registry repository.

        :param bundle_dir: Path to generated local directory that contains the bundle.
        :param namespace: Namespace that contains the repository for the application.
        :param repository: Repository name of the application described by the bundle.
        :param release: Release version of the bundle.
        :param auth_token: Authentication token used to push to Quay.io.
        """
        logger.info('Generating 64 bit bundle and pushing to app registry.')
        base64_bundle = self._create_base64_bundle(bundle_dir, repository)
        self._push_to_registry(namespace, repository, release, base64_bundle, auth_token)

    def _create_base64_bundle(self, bundle_dir, repository):
        with TemporaryDirectory() as temp_dir:
            tarfile_name = "%s/%s.tar.gz" % (temp_dir, repository)
            with tarfile.open(tarfile_name, "w:gz") as tar:
                tar.add(bundle_dir, "")
            with open(tarfile_name, "rb") as tarball:
                result = tarball.read()
            result64 = base64.b64encode(result).decode("utf-8")
            return result64

    def _push_to_registry(self, namespace, repository, release, bundle, auth_token):
        push_uri = 'https://quay.io/cnr/api/v1/packages/%s/%s' % (namespace, repository)
        logger.info('Pushing bundle to %s' % push_uri)
        headers = {'Content-Type': 'application/json', 'Authorization': auth_token}
        json = {'blob': bundle, 'release': release, "media_type": "helm"}
        r = requests.post(push_uri, json=json, headers=headers)
        if r.status_code != 200:
            logger.error(r.text)

            msg = 'Failed to get error details'
            try:
                error = r.json()['error']
                msg = "Quay.io answered: '{}' ({})".format(
                    error['code'], error['message'])
            except Exception:
                pass

            raise ValueError("Failed to push to app registry: {}".format(msg))



general_required_fields = [
    "spec"
]

metadata_required_fields = [
    {
        "field": "annotations",
        "description": "Without this field, your operator will be missing "
                       "essential data for the UI. Please add this field to the "
                       "CSV and run validation again to see what subfields "
                       "are required.",
        "required": True
    }
]

metadata_annotations_required_fields = [
    {
        "field": "description",
        "description": "Without this field, the description displayed in "
                       "the tiles of the UI will be a truncated "
                       "version of spec.description.",
        "required": True
    },
    {
        "field": "categories",
        "description": "Without this field, the operator will be "
                       "categorized as Other.",
        "required": False
    },
    {
        "field": "capabilities",
        "description": "Without this field, the operator will be assigned "
                       "the basic install capability - you can read more "
                       "about operator maturity models here "
                       "https://www.operatorhub.io/getting-started#How-do-I-start-writing-an-Operator?.",  # noqa
        "required": True
    },
    {
        "field": "repository",
        "description": "Without this field, the link to the operator source "
                       "code will not be displayed in the UI.",
        "required": False
    },
    {
        "field": "createdAt",
        "description": "Without this field, the time stamp at which the "
                       "operator was created will not be displayed in the UI.",
        "required": False
    },
    {
        "field": "containerImage",
        "description": "Without this field, the link to the operator "
                       "image will not be displayed in the UI.",
        "required": True
    },
    {
        "field": "alm-examples",
        "description": "Without this field, users will not have examples "
                       "of how to write Custom Resources for the operator.",
        "required": False
    }
]

spec_required_fields = [
    {
        "field": "displayName",
        "description": "Without this field, the name of the operator for "
                       "both the tile and the details view will default "
                       "to metadata.name.",
        "required": True
    },
    {
        "field": "description",
        "description": "Without this field, the description in the details "
                       "view of the operator will default to "
                       "metadata.annotations.description.",
        "required": True
    },
    {
        "field": "version",
        "description": "Without this field, the version in the details "
                       "view of the operator will be empty.",
        "required": True
    },
    {
        "field": "links",
        "description": "Without this field, no links will be displayed "
                       "in the details page side panel. You can for "
                       "example link to some additional Documentation, "
                       "related Blogs or Repositories.",
        "required": True
    },
    {
        "field": "icon",
        "description": "Without this field, the operator will display "
                       "a default operator framework icon.",
        "required": False
    },
    {
        "field": "provider",
        "description": "Without this field, users will not be able "
                       "to filter for provider.",
        "required": True
    },
    {
        "field": "maintainers",
        "description": "Without this field, the operator details page "
                       "will not display the name and contact for users "
                       "to get support in using the operator. "
                       "The field should be a yaml list of name & email pairs.",
        "required": True
    },
    {
        "field": "customresourcedefinitions",
        "description": "Without this field, the operator details page will "
                       "not display any CRDs the operator needs "
                       "in order to function.",
        "required": False
    },
    {
        "field": "installModes",
        "description": "Without this field, OLM will not be able "
                       "to deploy your operator.",
        "required": True
    }
]



from setuptools import setup

with open('README.md', 'r') as f:
    long_description = f.read()

tests_require = [
  'pytest',
  'pytest-cov',
  'testfixtures',
]

setup(
  name='operator-courier',
  packages=['operatorcourier'],
  version='1.2.0',
  description='Build, verify and push operators',
  author='AOS Marketplace',
  author_email='aos-marketplace@redhat.com',
  url='https://github.com/operator-framework/operator-courier',
  entry_points={
    'console_scripts': ['operator-courier=operatorcourier.cli:main'],
  },
  keywords=['operator', 'courier'],
  install_requires=[
    'pyyaml',
    'requests',
    'regex',
    'validators'
  ],
  setup_requires=['pytest-runner'],
  tests_require=tests_require,
  extras_require={
    'test': tests_require,
  },
  long_description=long_description,
  long_description_content_type="text/markdown",
  license="Apache License 2.0",
  include_package_data=True,
)



import os
from filecmp import dircmp
from tempfile import TemporaryDirectory
from operatorcourier.nest import nest_bundles


def test_nest_default():
    with TemporaryDirectory() as registry_dir:
        expected_result = "tests/test_files/bundles/nest/bundle1result"
        folder_to_nest = "tests/test_files/bundles/nest/bundle1"

        yaml_files = []
        for filename in os.listdir(folder_to_nest):
            with open(folder_to_nest + "/" + filename) as f:
                yaml_files.append(f.read())

        with TemporaryDirectory() as temp_dir:
            nest_bundles(yaml_files, registry_dir, temp_dir)

        dcmp = dircmp(registry_dir, expected_result)
        assert(len(dcmp.diff_files) == 0)
